/******************************************************************************
Utility to convert a LTP flow to a TCP flow for LTP analysis via tcptrace.
LTP Header manipulation functions

Copyright (C) 2013  Samuel Jero <sj323707@ohio.edu>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Author: Samuel Jero <sj323707@ohio.edu>
Date: 06/2013

Notes:
	1)Only handles one LTP "connection". There isn't a good way to separate
		different LTP "connections" from new sessions of the same "connection".
		Use Tcpdump filters to separate connections. Libpcap filtering could also
		be added in ltp2tcp.
	2)Uses some special types from Linux (u_char, u_int32_t)
******************************************************************************/
#include "ltp.h"


/*
 * SDNV has a zero in high-order bit position of last byte. The high-order
 * bit of all preceding bytes is set to one. This returns the numeric value
 * in an integer and sets the value of the second argument to the number of
 * bytes used to code the SDNV. A -1 is returned if the evaluation fails
 * (value exceeds maximum for signed integer). 0 is an acceptable value.
 */

#define SDNV_MASK	0x7f

/*2rd arg is number of bytes in field (returned)*/
int evaluate_sdnv(const u_char* loc, int *bytecount)
{
    int value = 0;
    u_char curbyte;

    *bytecount = 0;

    /*
     * Get 1st byte and continue to get them while high-order bit is 1
     */

    while((curbyte = *(loc++)) & ~SDNV_MASK) {
	if(*bytecount >= (int) sizeof(int)) {
	    *bytecount = 0;
	    return -1;
	}
	value = value << 7;
	value |= (curbyte & SDNV_MASK);
	++*bytecount;
    }

    /*
     * Add in the byte whose high-order bit is 0 (last one)
     */

    value = value << 7;
    value |= (curbyte & SDNV_MASK);
    ++*bytecount;
    return value;
}

/* Special Function to evaluate 64 bit SDNVs */
long long evaluate_sdnv_64(const u_char* loc, int *bytecount)
{
    long long value = 0;
    u_char curbyte;

    *bytecount = 0;

    /*
     * Get 1st byte and continue to get them while high-order bit is 1
     */

    while((curbyte = *(loc++)) & ~SDNV_MASK) {
	if(*bytecount >= (int) sizeof(long long)) {
	    *bytecount = 0;
	    return -1;
	}
	value = value<< 7;
	value |= (curbyte & SDNV_MASK);
	++*bytecount;
    }

    /*
     * Add in the byte whose high-order bit is 0 (last one)
     */

    value = value << 7;
    value |= (curbyte & SDNV_MASK);
    ++*bytecount;
    return value;
}
